@@ -23,13 +23,10 @@ RELEASE_FLAG=${RELEASE_FLAG:=release}
 
 ./dev/build-ballista-executables.sh
 
-docker compose build
-
 . ./dev/build-set-env.sh
-docker build -t "apache/arrow-ballista-standalone:$BALLISTA_VERSION" -f dev/docker/ballista-standalone.Dockerfile .
 
-docker tag ballista-executor "apache/arrow-ballista-executor:$BALLISTA_VERSION"
-docker tag ballista-scheduler "apache/arrow-ballista-scheduler:$BALLISTA_VERSION"
-docker tag ballista-benchmarks "apache/arrow-ballista-benchmarks:$BALLISTA_VERSION"
-
-docker build -t "apache/arrow-ballista-cli:$BALLISTA_VERSION" -f dev/docker/ballista-cli.Dockerfile .
+docker build -t "apache/arrow-ballista-standalone:latest" -f dev/docker/ballista-standalone.Dockerfile .
+docker build -t "apache/arrow-ballista-scheduler:latest" -f dev/docker/ballista-scheduler.Dockerfile .
+docker build -t "apache/arrow-ballista-executor:latest" -f dev/docker/ballista-executor.Dockerfile .
+docker build -t "apache/arrow-ballista-cli:latest" -f dev/docker/ballista-cli.Dockerfile .
+docker build -t "apache/arrow-ballista-benchmarks:latest" -f dev/docker/ballista-benchmarks.Dockerfile .
